/********************************************************/
/*	SAMPLE PROGRAM 4 FOR MSA0654-MEAUST BOARD	*/
/*	FILENAME: sample4.c				*/
/*	WRITTEN BY: Alan Suyko  28/7/99			*/
/*	DESCRIPTION:  					*/
/*	-- Demonstrates use of A-D converter by reading	*/
/*	voltage across potentiometer R24 and displaying	*/
/*	result on the LEDs. 				*/
/*	-- Use ADtrg# line to start conversion.		*/
/*	-- Uses Timer A1 to multiplex refresh of LED1 	*/
/*	and LED2.					*/
/*							*/
/*	Copyright 1999 by MITSUBISHI ELECTRIC AUSTRALIA	*/
/*	All rights reserved. See Terms and Conditions	*/
/*	document regarding the use of this sample	*/
/*	program.					*/
/********************************************************/ 	

#include "sfr62.h"
#include "leddata.h"
void main(void);
void initport(void);
void initADC(void);
void setADCint(void);
void ADCint(void);
void initTimerA1(void);
void startTimerA1(void);
void setTimerA1int(void);
void TimerA1int(void);

#pragma INTERRUPT ADCint
#pragma INTERRUPT TimerA1int

unsigned char pot_rdg = 0;

void main(void)
{
	initport();
	initADC();
	initTimerA1();
	setADCint();
	setTimerA1int();
	startTimerA1();
	while(1);	// endless loop
}

void initport(void)
{
	pd0 = 0xFF;	// output mode 
	pd1 = 0xFF;	// output mode 
	pu00 = 0;	// no pull up for P0_0 to P0_3 
	pu01 = 0;	// no pull up for P0_4 to P0_7 
	pu02 = 0;	// no pull up for P1_0 to P1_3 
	p1 = 0xFD;	// enable LED2 and disable LED1
	p0 = 0xFF;	// turn off all segments
}

void initADC(void)
{
    adcon2 = 0x01;          // ---- ---X 
                            //         +- 0: without sample and hold
                            //            1: with sample and hold


    adcon0 = 0x60;          // X0X0 0000- AN0 SELECTED
                            // |||| +---- must be fixed to 0 in one shot mode
                            // |||+------ must be fixed to 0 in one shot mode
                            // ||+------- trigger select bit
                            // ||         0: software trigger
                            // ||         1: external trigger (ADtrg, P10.0)
                            // |+-------- 0: A-D conversion disabled
			    // |          1: A-D conversion enabled
                            // +--------- frequency select bit 0:
                            //            0: fad/4 is selected
                            //            1: fad/2 is selected
                            //            please look at bit 4 of ADCON1
                            //            (ADCON1.4) also)

// NOTE:  
//	With external trigger, A-D conversion starts when the A-D conversion
//	start flag is "1" and the ADtrg#/P9.7 input changes from "H" to "L"

     adcon1 = 0x30;         // XX1X X0--
                            // |||| |+--- must be fixed to 0 in one shot mode
                            // |||| +---- 0: 8-bit mode 
                            // ||||       1: 10-bit mode
                            // |||+------ frequency select bit 1
                            // |||        0: fad/2 or fad/4 is selected
                            // |||           please look at bit 7 of ADCON0
                            // |||        1: fad is selected 
                            // ||+------- 1: Vref connected (must be fixed to
                            // ||            1 if A-D conversion is used)
                            // |+-------- ext. op amp connection mode bits
                            // +--------- ext. op amp connection mode bits
                            //            00: ANEX0 and ANEX1 are not used 
                            //            01: ANEX0 input is A-D converted
                            //            10: ANEX1 input is A-D converted
                            //            11: external op amp connection mode
}


void setADCint(void)
{
    adic = 0x01;            // ---- XXXX
                            //      ||||
                            //      |||+-- Interupt priority level select bit
                            //      ||+--- Interupt priority level select bit
                            //      |+---- Interupt priority level select bit
                            //      |      000: Level 0 (interrupt disabled)
                            //      |      001: Level 1
                            //      |      010: Level 2
                            //      |      011: Level 3
                            //      |      100: Level 4
                            //      |      101: Level 5
                            //      |      110: Level 6
                            //      |      111: Level 7
                            //      +----- Interupt request bit 
                            //             0: Interrupt not requested
                            //             1: Interrupt requested
}


void ADCint(void)		// interrupt function
{	
	asm("fset I");		// enable interrupts
    	pot_rdg = ad0l;		// read low-order byte of AD0 register
}


void initTimerA1(void)
{       		
    	ta1mr = 0x80;       // XX0X XX00 
                            // |||| |||+- must always be 0 in timer mode
                            // |||| ||+-- must always be 0 in timer mode
                            // |||| |+--- 0: pulse is not output at pin TA1out
                            // |||| |     1: pulse is output at pin TA1out
                            // |||| |        TA0out is automatically  output
                            // |||| +---- 0: gate function: timer counts only 
                            // ||||          when TA0in is held "L"
                            // ||||       1: gate function: timer counts only
                            // ||||          when TA0in is held "H"
                            // |||+------ 0: gate function not available
                            // |||        1: gate function available
                            // ||+------- must always be 0 in timer mode
                            // |+-------- count source select bits:
                            // +--------- count source select bits:
                            //            00:  f1
                            //            01:  f8
                            //            10:  f32
                            //            11:  fc32

    	ta1 = 0x1388;	    // Set up Timer A1 Reload Register for 10 msec interval
			    // for a LED refresh rate of 100Hz
}


void startTimerA1(void)
{
    tabsr |= 0x02;          //	1: start timer A1 (count flag)
}


void setTimerA1int(void)
{
    ta1ic = 0x01;           // ---- XXXX
                            //      ||||
                            //      |||+-- Interupt priority level select bit
                            //      ||+--- Interupt priority level select bit
                            //      |+---- Interupt priority level select bit
                            //      |      000: Level 0 (interrupt disabled)
                            //      |      001: Level 1
                            //      |      010: Level 2
                            //      |      011: Level 3
                            //      |      100: Level 4
                            //      |      101: Level 5
                            //      |      110: Level 6
                            //      |      111: Level 7
                            //      +----- Interupt request bit 
                            //             0: Interrupt not requested
                            //             1: Interrupt requested
}


void TimerA1int(void)		// interrupt function
{	
	static unsigned char led_select = 0;
	unsigned char offset, pot_rdg_scaled;

	asm("fset I");		// enable interrupts
	if (led_select == 0)
		{
		led_select = 1;
		p1 = 0xFD;		// enable LED2 and disable LED1 
		pot_rdg_scaled = (pot_rdg * 5 * 10)/255;  // 5.0V full scale		
		offset = pot_rdg_scaled % 10;	
		p0 = leddigit[offset];	// display remainder
		}
	else
		{
		led_select = 0;
		p1 = 0xFE;		// disable LED2 and enable LED1 
		pot_rdg_scaled = (pot_rdg * 5 * 10)/255;  // 5.0V full scale			
		offset = pot_rdg_scaled / 10;	
		p0 = leddigit[offset];	// display quotient
		p0 &= 0x7F;		// turn on DP segment
		}
}


